//
// Name: FTLPT.PRG
// Project: Fischer Techniek interfacing
// Description: ft lpt 30520 interface driver
// Language: Clipper 5.2
// Author: Rene Nooteboom
// Date created: 21-08-2005
// Date updated: 29-08-2005
//
//  default values :
//
//  parallel port  I/O   IRQ
//
//   LPT1         378H    7
//   LPT2         278H    5
//   LPT3         3BCH    -
//   LPT4         2BCH    -
//
//  example ftlpt.ini file :  //
//                            LPT3_IO = h378
//                            LPT2_IO = 632
//                            //
//
#include "generic.ch"
#include "noot.ch"
#include "ft.ch"
#include "ftlpt.ch"
//
STATIC cHexLptIO_:={"378","278","3BC","2BC"} // max 4 lptports
STATIC nLptStatusReg_ := {889,633,957,701} // offset 1 from dataregister
STATIC nLptControlReg_ := {890,634,958,702} // offset 2 from dataregister
STATIC lLptUsed_ := {FALSE,FALSE,FALSE,FALSE},nMotorByte_:={0,0,0,0}
//
FUNCTION FtLInit(nLptPort,xIOAddress,lAskValues)
//
// all arguments optional,xIOAddress can be numeric decimal or a hex-string
//
 LOCAL cmsg,getlist:={},nStatusRegIO,nControlRegIO
 DEFAULT nLptPort TO 1,lAskValues TO TRUE
 IF xIOAddress == NIL
  xIOAddress := FtLGetIOAddress(nLptPort)
 ENDIF
 IF IS_NUM(xIOAddress) // convert to hex
  xIOAddress := padl(alltrim(ntoc(xIOAddress,16,3)),3,"0")
 ENDIF
 IF lAskValues
  save_env({9,5,12,40})
  color_set(CR_GETS)
  @ 9,5 CLEAR TO 12,40
  @ 9,5 TO 12,40
  @ 10,6 SAY "LPT-port(1 to 4) :"
  @ 11,6 SAY "IO-Address(hex)  :"
  @ 10,30 GET nLptPort PICTURE "9" VALID IS_BETW(nLptPort,1,4)
  @ 11,30 GETX xIOAddress MESSAGE "hexadecimal" PICTURE "@! XXX" ;
   VALID IS_HEX(xIOAddress) ERROR "not hex"
  READ
  rest_env()
  IF lastkey() == K_ESC
   nLptPort := -1
  ENDIF
 ENDIF
 IF nLptPort > 0
  cHexLptIO_[nLptPort] := xIOAddress
  nStatusRegIO := cton(xIOAddress,16) + 1
  nControlRegIO := nStatusRegIO + 1
  nLptStatusReg_[nLptPort] := nStatusRegIO
  nLptControlReg_[nLptPort] := nControlRegIO
  lLptUsed_[nLptPort] := TRUE
  FtLOutput(nLptPort,0) // switch all motors off
  cmsg := "FtLInit(): initialized LPT" + str(nLptPort,1,0) + ", IO: " + ;
   xIOAddress + "h"
  rowmessage(cmsg,,,CR_MESSAGE,2)
  FtLShowUsedIF()
 ELSE
  nLptPort := -1
 ENDIF
RETURN nLptPort
//
PROCEDURE FtLClose(nLptPort)
 DEFAULT nLptPort TO 1
 IF nLptPort > 0 .AND. lLptUsed_[nLptPort]
  FtLOutput(nLptPort,0)
  lLptUsed_[nLptPort] := FALSE
 ENDIF
RETURN
//
FUNCTION FtLInput(nLptPort)
 LOCAL nBitNr,nTestByte,nByteToPort,nInputByte:=0,cmsg
 DEFAULT nLptPort TO 1
 nByteToPort := 50 // =32h=0011 0010 LOAD IN HIGH
 outbyte(cHexLptIO_[nLptPort],nByteToPort)
 nByteToPort := setbit(nByteToPort,CLOCKSIGNAL) // set clock high 0011 1010
 outbyte(cHexLptIO_[nLptPort],nByteToPort)
 // create input byte first bit is E8,then E7....E1
 FOR nBitNr := 8 TO 1 STEP -1
  nTestByte := inbyte(nLptStatusReg_[nLptPort]) // read status register
  IF isbit(nTestByte,DATA_IN) // test bit DATA_IN line
   nInputByte := setbit(nInputByte,nBitNr)
  ENDIF
  nByteToPort := RUHEBYTE  // static byte pattern
  outbyte(cHexLptIO_[nLptPort],nByteToPort)
  nByteToPort := setbit(nByteToPort,CLOCKSIGNAL) // CLOCK HIGH
  IF !outbyte(cHexLptIO_[nLptPort],nByteToPort)
   nInputByte := 0
   cmsg := "FtLInput(): Error writing to port:" + ;
    cHexLptIO_[nLptPort] + "h"
   rowmessage(cmsg,,,CR_WARNING,4,TRUE)
   EXIT
  ENDIF
 NEXT
RETURN nInputByte
//
PROCEDURE FtLOutput(nLptPort,nByte)
 LOCAL nBitNr,nByteToPort
 DEFAULT nLptport TO 1
 FOR nBitNr := 8 TO 1 STEP -1  // MSB(M4) comes first !
  nByteToPort := RUHEBYTE // static byte pattern
  IF isbit(nByte,nBitNr)
   nByteToPort := setbit(nByteToPort,DATA_OUT)
  ENDIF
  outbyte(cHexLptIO_[nLptPort],nByteToPort)
  nByteToPort := setbit(nByteToPort,CLOCKSIGNAL) // clock low->high
  outbyte(cHexLptIO_[nLptport],nByteToPort)
 NEXT
 nByteToPort := 57 // 39h=0011 1001 load out high
 outbyte(cHexLptIO_[nLptPort],nByteToPort)
 nMotorByte_[nLptport] := nByte
RETURN
//
// AD-waarde hangt af van speed processor
// hoe sneller de cpu hoe hoger de nAnalogValue
// dus altijd per toepassing yken
//
FUNCTION FtLAD(nLptPort,nAnalogInput) // EX=1,EY=2
 LOCAL nByteToPort,nTestByte,nAnalogValue:=0,lCountIn
 DEFAULT nLptPort TO 1,nAnalogInput TO EX
 IF nAnalogInput == EY
  nByteToPort := 144 // 90h = 1001 0000
 ELSE
  nByteToPort := 160 // A0h = 1010 0000
 ENDIF
 outbyte(cHexLptIO_[nLptPort],nByteToPort)
 nByteToPort := 56 // 38h = set clock 0011 1000
 outbyte(cHexLptIO_[nLptPort],nByteToPort)
 lCountIn := TRUE
 DO WHILE lCountIn
  lCountIn := isbit(inbyte(nLptStatusReg_[nLptPort]),COUNT_IN)
  nAnalogValue++
 ENDDO
RETURN nAnalogValue
//
FUNCTION FtLSensor(nLptPort,nE1toE8)
 LOCAL nInput:=0,lSensor:=FALSE
 nInput := FtLInput(nLptPort)
 lSensor := isbit(nInput,nE1toE8)
RETURN lSensor
//
PROCEDURE FtLMotorsOff(nLptPort)
 FtLOutput(nLptPort,0)
RETURN
//
PROCEDURE FtLMotor(nLptPort,nMotorNr,nDirection) // MON,MCW,MCCW,MOFF
 LOCAL nOutputByte
 DEFAULT nLptPort TO 1
 nOutputByte := nMotorByte_[nLptPort]
 DO CASE
  CASE nMotorNr == 1
   nOutputByte := numand(nOutputByte,"FC") // 1111 1100
   nOutputByte := numor(nOutputByte,nDirection) // no shift
  CASE nMotorNr == 2
   nOutputByte := numand(nOutputByte,"F3") // 1111 0011
   nOutputByte := numor(nOutputByte,nDirection*4) // left shift 2 bits
  CASE nMotorNr == 3
   nOutputByte := numand(nOutputByte,"CF") // 1100 1111
   nOutputByte := numor(nOutputByte,nDirection*16) // left shift 4 bits
  CASE nMotorNr == 4
   nOutputByte := numand(nOutputByte,"3F") // 0011 1111
   nOutputByte := numor(nOutputByte,nDirection*64) // left shift 6 bits
  OTHERWISE
 ENDCASE
 FtLOutput(nLptPort,nOutputByte)
RETURN
//
PROCEDURE FtLButton(nLptPort,nE1toE8)
 DO WHILE !FtLSensor(nLptPort,nE1toE8)
 ENDDO
RETURN
//
PROCEDURE FtLButtonPushed(nLptPort,nE1toE8)
 DO WHILE FtLSensor(nLptPort,nE1toE8)
 ENDDO
RETURN
//
PROCEDURE FtLMotorLeft(nLptPort,nMotorNr)
 FtLMotor(nLptPort,nMotorNr,MCCW)
RETURN
//
PROCEDURE FtLMotorRight(nLptPort,nMotorNr)
 FtLMotor(nLptPort,nMotorNr,MCW)
RETURN
//
PROCEDURE FtLMotorOff(nLptPort,nMotorNr)
 FtLMotor(nLptPort,nMotorNr,MOFF)
RETURN
//
PROCEDURE FtLMotorOn(nLptPort,nMotorNr)
 FtLMotor(nLptPort,nMotorNr,MON)
RETURN
//
STATIC FUNCTION FtLGetIOAddress(nLptPort)
 LOCAL xIOAddress:=NIL,csearch
 DEFAULT nLptPort TO 1
 IF file("ftlpt.ini")
  readinival("ftlpt.ini")
  csearch := "LPT" + str(nLptPort,1,0) + "_IO" // example: LPT3_IO = h3BC
  IF ini_exist(csearch)
   IF !empty(xIOAddress := ini_value(csearch))
    DO CASE
     CASE IS_NUM(xIOAddress)  // assume decimal number
     CASE IS_CHAR(xIOAddress)
      IF upper(left(xIOAddress,1)) == "H" // hex number follows "Hxxx"
       xIOAddress := padl(alltrim(substr(xIOAddress,2)),3,"0")
      ELSE
       IF !IS_HEX(xIOAddress)
        xIOAddress := NIL
       ELSE
        xIOAddress := padl(alltrim(xIOAddress),3,"0")
       ENDIF
       xIOAddress := NIL
      ENDIF
     OTHERWISE
      xIOAddress := NIL
    ENDCASE
   ENDIF
  ENDIF
  clearinival()
 ENDIF
 IF empty(xIOAddress)
  DO CASE
   CASE nLptPort == 1
    xIOAddress := "378"
   CASE nLptPort == 2
    xIOAddress := "278"
   CASE nLptPort == 3
    xIOAddress := "3BC"
   CASE nLptPort == 4
    xIOAddress := "2BC"
   OTHERWISE
    xIOAddress := "378"
  ENDCASE
 ENDIF
RETURN xIOAddress
//
STATIC PROCEDURE FtLShowUsedIF()
 LOCAL cmsg:="Ft interface uses:",i
 FOR i := 1 TO 4
  IF lLptUsed_[i]
   cmsg += " LPT" + str(i,1,0) + ", IO:" + cHexLptIO_[i] + "h"
  ENDIF
 NEXT
 rowmessage(cmsg,,,CR_MESSAGE,3)
RETURN
//
// sec delay but keeps interfaces alive
//
PROCEDURE FtLDelay(nDelayTime)
 LOCAL nBeginTime := seconds(),i
 DO WHILE TRUE
  IF nBeginTime > seconds()
   nBeginTime -= 86400
  ENDIF
  FOR i:=1 TO 4
   IF lLptUsed_[i]
    FtLInput(i)
   ENDIF
  NEXT i
  IF (seconds() - nBeginTime) > nDelayTime
   EXIT
  ENDIF
 ENDDO
RETURN
//
FUNCTION FtLHexIO(nLptPort)
 LOCAL cHexIO:="---"
 DEFAULT nLptPort TO 1
 IF nLptPort > 0 .AND. lLptUsed_[nLptPort]
  cHexIO := cHexLptIO_[nLptPort]
 ENDIF
RETURN cHexIO
//
// einde FtLpt.prg
//
